// Copyright (C) 2008
// by all contributors.

// This file is part of SCNSL.

// SCNSL is free software:
// you can redistribute it and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option)
// any later version.

// SCNSL is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY;
// without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.
// See the GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with SCNSL,
// in a file named LICENSE.txt.
// If not, see <http://www.gnu.org/licenses/>.



/// @file
/// A Traffic task.

#include <sstream>

#include "Traffic_if_t.hh"


using Scnsl::Traffic::Traffic_if_t;


// ////////////////////////////////////////////////////////////////
// Destructor.
// ////////////////////////////////////////////////////////////////

Traffic_if_t::~Traffic_if_t()
    throw ()
{
	// to do
}


// ////////////////////////////////////////////////////////////////
// Constructor.
// ////////////////////////////////////////////////////////////////

Traffic_if_t::Traffic_if_t( sc_core::sc_module_name modulename,
                            const task_id_t id,
                            Scnsl::Core::Node_t * n,
                            const size_t proxies,
							label_t label
						  )
    throw ( std::invalid_argument ):
    // Parents:
    Scnsl::Tlm::TlmTask_if_t( modulename, id, n, proxies ),
    // Fields:
	_label( label )
{

}


// ////////////////////////////////////////////////////////////////
// Interface methods.
// ////////////////////////////////////////////////////////////////

void Traffic_if_t::setLabel( label_t label )
	throw()
{
	_label = label;
}


void Traffic_if_t::b_transport( tlm::tlm_generic_payload & p, sc_core::sc_time & t )
{
	t=t;
	char result;
	bool c;
    size_t s;

	if(p.get_command() == Scnsl::Tlm::PACKET_COMMAND)
	{
    	result = static_cast< char >( * p.get_data_ptr() );
        s = p.get_data_length();

        // Since we are tracing a complex message,
        // we wrap the log with the macro, to avoid
        // unuseful overhead.
#if ( SCNSL_LOG >= 1 )
        std::stringstream ss;
        ss << "data received: " << result << " bytes: " << s;
        SCNSL_TRACE_LOG( 1, ss.str().c_str() );
#endif

	}

	if(p.get_command() == Scnsl::Tlm::CARRIER_COMMAND)
	{
		c = * p.get_data_ptr();

        // Since we are tracing a complex message,
        // we wrap the log with the macro, to avoid
        // unuseful overhead.
#if ( SCNSL_LOG >= 1 )
        std::stringstream ss;
        ss << "carrier: " << c;
        SCNSL_TRACE_LOG( 1, ss.str().c_str() );
#endif
	}
}
